@@ -1,6 +1,6 @@
 from collections import OrderedDict
 from copy import deepcopy
-import warnings
+import logging
 
 import numpy as np
 from astropy import units as u
@@ -9,20 +9,39 @@ from astropy.table import Table, vstack
 
 from .surface import SpectralSurface
 from ..utils import real_colname, insert_into_ordereddict, quantify, \
-    change_table_entry, convert_table_comments_to_dict
+    change_table_entry, convert_table_comments_to_dict, from_currsys
 
 
 def combine_emissions(tbl, surfaces, row_indexes, etendue, use_area=False):
+    '''Combine thermal emission from a series of surfaces
+
+    The function traces thermal emission through an optical system, taking
+    into account the finite reflectivities/transmissivities and emissivities
+    of the surfaces. The function assumes that etendue is conserved through
+    the system, i.e. surfaces are neither over- nor undersized.
+
+    Parameters
+    ----------
+    tbl : astropy Table
+        Required columns are `name` and `action` (reflection or transmission)
+    surfaces: OrderedDict of SpectralSurface
+        Keys are the names from tbl, values are of type `SpectralSurface`
+    row_indexes : list of int
+        Rows of tbl (i.e. surfaces) to combine
+    etendue, use_area : not needed  (TODO: remove)
+
+    Returns
+    -------
+    SourceSpectrum
+    '''
     if len(tbl) == 0:
         return None
 
-    etendue = quantify(etendue, "m2 arcsec2")
-
     r_name = real_colname("name", tbl.colnames)
     r_action = real_colname("action", tbl.colnames)
 
     emission = None
-    for ii, row_num in enumerate(row_indexes):
+    for row_num in row_indexes:
         row = tbl[row_num]
         surf = surfaces[row[r_name]]
         action_attr = row[r_action]
@@ -32,34 +51,15 @@ def combine_emissions(tbl, surfaces, row_indexes, etendue, use_area=False):
             if emission is not None:
                 emission = emission * surf_throughput
 
-            area = surf.area
-            if area is not None:
-                surf_emission = surf.emission       # PHOTLAM * arcsec**-2
-                surf_eff_area = area * np.cos(surf.mirror_angle)
-                surf_eff_solid_angle = (etendue / surf_eff_area).to(u.arcsec**2)
-                surf_emission *= surf_eff_solid_angle.value
-
-                if use_area:
-                    surf_emission *= area.to(u.cm**2).value
-                    surf_emission.meta["use_area"] = use_area
-
-                msg = "Etendue scale factor applied. Effective pixel solid " \
-                      "angle for surface is {}".format(surf_eff_solid_angle)
-                surf_emission.meta["solid_angle"] = None
-                surf_emission.meta["history"] += [msg]
-
-                if ii == 0:
-                    emission = deepcopy(surf_emission)
-                else:
-                    emission = emission + surf_emission
-
+            if emission is None:
+                emission = deepcopy(surf.emission)
             else:
-                warnings.warn('Ignoring emission from surface: "{}". Area came '
-                              'back as "None"'.format(surf.meta["name"]))
+                emission = emission + surf.emission
 
     return emission
 
 
+
 def combine_throughputs(tbl, surfaces, rows_indexes):
     if len(tbl) == 0:
         return None
@@ -100,6 +100,7 @@ def combine_tables(new_tables, old_table=None, prepend=False):
         if old_table is None:
             old_table = new_table
         else:
+            new_table = from_currsys(new_table)
             if prepend:
                 old_table = vstack([new_table, old_table])
             else:
@@ -134,7 +135,7 @@ def add_surface_to_table(tbl, surf, name, position, silent=True):
                                          position=position)
         else:
             if not silent:
-                warnings.warn("{} was not found in the meta dictionary of {}. "
+                logging.warning("{} was not found in the meta dictionary of {}. "
                               "This could cause problems".format(colname, name))
 
     colname = real_colname("name", new_tbl.colnames)
@@ -166,4 +167,3 @@ def make_surface_from_row(row, **kwargs):
     surface = SpectralSurface(**kwargs)
 
     return surface
-
